from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
import qgis.utils 
from Ui_GeorefExport import Ui_GeorefExport

import os, sys, time,  math, subprocess, platform, datetime
from osgeo import gdal, ogr
from osgeo.gdalconst import *

import numpy as np
import fnmatch



class GeorefExportDialog(QDialog, Ui_GeorefExport):

    def __init__(self, iface):
        QDialog.__init__(self)
        self.iface = iface
        self.setupUi(self)
        self.NOVALUE=-9999
        self.connect(self.btnOutputDir, SIGNAL("clicked()"), self.outDirFun)
        self.connect(self.buttonBox, SIGNAL("accepted()"),self.accept)
        QObject.connect(self.buttonBox, SIGNAL("rejected()"),self, SLOT("reject()"))
	QObject.connect(self.buttonBox, SIGNAL("helpRequested()"),self.call_help)

        mapCanvas = self.iface.mapCanvas()
	# get projecr epsg
	self.mapRenderer = iface.mapCanvas().mapRenderer()
	if QGis.QGIS_VERSION_INT < 10900:
		projectCRS=self.mapRenderer.destinationSrs().epsg()
	else:
		projectCRS=int(self.mapRenderer.destinationCrs().postgisSrid())
	self.lineEditSourceSrs.clear()	
	self.lineEditSourceSrs.insert(str(projectCRS))	
	
	self.drvDescription={}
	self.drvExt={}
	gdal.AllRegister()	
#	for i in range(0, gdal.GetDriverCount()):
#		drv = gdal.GetDriver(i)
	drvCodeSelected=["PDF","GTiff","JPEG","PNG","JPEG2000","ECW","JP2ECW","JP2MrSID","JP2OpenJPEG"]
	for i in range(0, len(drvCodeSelected)):
		print(gdal.__version__)
		drv = gdal.GetDriverByName(drvCodeSelected[i])
		if drv:
			drv_meta = drv.GetMetadata()
			if 'DMD_EXTENSION' in drv_meta: 
			   #\ and drv_meta.has_key(gdal.DCAP_CREATE) and drv_meta[gdal.DCAP_CREATE] == 'YES':
				drvLName="["+drvCodeSelected[i]+"] "+drv.LongName				
				self.drvDescription[drvLName]=drv.GetDescription()
				self.drvExt[drvLName]=drv_meta['DMD_EXTENSION']
				self.cmboxImgFormat.addItem(drvLName)	
	
	# get map rectangle
	mapRect = self.mapRenderer.extent()
	self.xStart = mapRect.xMinimum()
	self.xEnd = mapRect.xMaximum()
	self.yStart = mapRect.yMinimum()
	self.yEnd = mapRect.yMaximum()
	self.metersPerPixel=0.0000
    def call_help(self):
	qgis.utils.showPluginHelp()

    def createWorldFile(self,fileName, mainScale, wFileExt):
	#>>Create World File
	mapRect = self.mapRenderer.extent()
	f = open(fileName + "." + wFileExt, 'w')
	f.write(str(mainScale) + '\n')
	f.write(str(0) + '\n')
	f.write(str(0) + '\n')
	f.write('-' + str(mainScale) + '\n')
	f.write(str(mapRect.xMinimum() + mainScale/2) + '\n')
	f.write(str(mapRect.yMaximum() - mainScale/2))
	f.close()

        
    def outDirFun(self):
        "Display file dialog for output directory"
        self.lineOutput.clear()
        dirName = QFileDialog.getExistingDirectory( self, 
                                                      self.tr("Select save directory"),
                                                      QDir.currentPath(), 
                                                      QFileDialog.ShowDirsOnly|
                                                      QFileDialog.ReadOnly )
	if QGis.QGIS_VERSION_INT < 10900:		
		if not dirName.isEmpty():
			self.lineOutput.clear()
			self.lineOutput.insert(dirName)
	else:
		if dirName:
			self.lineOutput.clear()
			self.lineOutput.insert(dirName)	
        return dirName

    
    def doImage(self,myImagePath,myImagepathNoExt):

	dpi = self.spinBoxDpi.value()
	myWidth = self.spinBoxWidth.value() #width in pixels

	self.metersPerPixel = (self.xEnd-self.xStart)/(myWidth)
	myHeight = int((self.yEnd - self.yStart)/self.metersPerPixel) 
	#print myWidth
	#print myHeight
	xPaperSize = myWidth/(dpi/25.4)
	yPaperSize = myHeight/(dpi/25.4)
	#self.textEdit.append("xPaperSize %f" % (xPaperSize))
	#self.textEdit.append("yPaperSize %f" % (yPaperSize))
	
	composition = QgsComposition(self.mapRenderer)
        composition.setPrintResolution(dpi)
        composition.setPaperSize(xPaperSize, yPaperSize)
        composition.setPlotStyle(QgsComposition.Print)
	#dpi = composition.printResolution()
        dpmm = dpi / 25.4 #get dots per mm
        # add a map to the composition
        composerMap = QgsComposerMap(composition,0,0,composition.paperWidth(),composition.paperHeight())
        composition.addItem(composerMap)
        # create output image and initialize it
        image = QImage(QSize(myWidth, myHeight), QImage.Format_ARGB32) #output image size
        image.setDotsPerMeterX(dpmm * 1000) #mm to meters
        image.setDotsPerMeterY(dpmm * 1000) #mm to meters
        image.fill(0)
        # render the composition
        imagePainter = QPainter(image)
        sourceArea = QRectF(0, 0, composition.paperWidth(), composition.paperHeight() )
        targetArea = QRectF(0, 0, myWidth, myHeight)
        composition.render(imagePainter, targetArea, sourceArea)
        imagePainter.end()
#	self.textEdit.append(imagepath)
        image.save(myImagePath , "tif")	
        self.createWorldFile(myImagepathNoExt, self.metersPerPixel, "tfw")



    def accept(self):
        # Called when "OK" button pressed
        
	self.textEdit.clear()
	inizio = datetime.datetime.now()
        self.textEdit.append("Starting...")
	outDir=self.lineOutput.text()
	if (outDir == ''):
		QMessageBox.critical(None,"Exiting gracefully","Output directory not defined %s!" % (outDir))
		return
	if (len(outDir.split(' ')) > 1):
		QMessageBox.critical(None,"Exiting gracefully","Output directory contains spaces %s!" % (outDir))
		return

        outImage=self.lineEditImageName.text()
	ssrs=self.lineEditSourceSrs.text()
	tmpimagepath=outDir+"/tmp_"+outImage+".tif"
	tmpimagepathNoExt=tmpimagepath[:-4]
	imagepathNoExt=outDir+"/"+outImage
	
	self.doImage(tmpimagepath,tmpimagepathNoExt)
	
	imgFormat=self.cmboxImgFormat.currentText()
	
	outputFormat=self.drvDescription[imgFormat]
	outputExt=self.drvExt[imgFormat]	

	# do translate...
	gdal_translateCMD=str("-of "+ outputFormat+" -a_srs epsg:"+ssrs+" "+tmpimagepath+" "+imagepathNoExt+"."+outputExt )
	self.textEdit.append("gdal_translate "+gdal_translateCMD)


    	processTranslate = QProcess( parent=None )
	if QGis.QGIS_VERSION_INT < 10900:
	    	processTranslate.start( "gdal_translate",QStringList() << gdal_translateCMD.split(" "), QIODevice.ReadOnly )
        else:
		processTranslate.start( "gdal_translate",gdal_translateCMD.split(" "), QIODevice.ReadOnly )
	arr = QByteArray()
    	if processTranslate.waitForFinished(-1):
    		arr = processTranslate.readAllStandardOutput()
    		processTranslate.close()
    	self.textEdit.append("gdal_translate:\n - "+ str(arr) )
    	
    	if os.path.isfile(imagepathNoExt+"."+outputExt) and self.checkWFile.isChecked():
    		wfext=outputExt[0]+outputExt[-1:]+"w"
    		self.createWorldFile(imagepathNoExt, self.metersPerPixel, wfext.lower())
    	
    	#remove tmp files
	os.remove(tmpimagepathNoExt+".tfw")
    	os.remove(tmpimagepath)
    	
	self.textEdit.append("")
	fine = datetime.datetime.now()
	self.textEdit.append( 'Starting time: '+ str(inizio) )
	self.textEdit.append( 'Ending time: '+ str(fine) )
	diff = fine-inizio
	self.textEdit.append( "Elapsed time: " + str(diff) )
	
	self.textEdit.append( "\nDone!\n")
	self.textEdit.append( "Go to " + outDir)
	self.textEdit.append( "to find exported map!!!")
        self.textEdit.append("")

